
#
# $Copyright (c) 2018 Veritas Technologies LLC.  All rights reserved. Veritas
# and the Veritas Logo are trademarks or registered trademarks of Veritas
# Technologies LLC or its affiliates in the U.S. and other countries. Other
# names may be trademarks of their respective owners.
#
# The Licensed Software and Documentation are deemed to be commercial computer
# software as defined in FAR 12.212 and subject to restricted rights as defined
# in FAR Section 52.227-19 "Commercial Computer Software - Restricted Rights" and
# DFARS 227.7202, et seq. "Commercial Computer Software and Commercial Computer
# Software Documentation," as applicable, and any successor regulations, whether
# delivered by Veritas as on premises or hosted services. Any use, modification,
# reproduction release, performance, display or disclosure of the Licensed Software
# and Documentation by the U.S. Government shall be solely in accordance with the
# terms of this Agreement. $

# $Revision: c42fbdc92cc5c484c79f7a537b20d1e9996afc3a $
# $Source: src/srdfsg/unix/StorageGroupAgent.pm $
#


package StorageGroupAgent;
use ArrayRep;
@ISA = ("ArrayRep");

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);
use POSIX qw(uname);
use File::Basename;
use ag_i18n_inc;
my $vcs_home=$VCSHOME;
my $vcs_log=$VCSLOG;
my $USERLANG=$ENV{"LANG"};
my $VXDISK = "/usr/sbin/vxdisk";
my ($OS, $DEVNULL) = ((uname())[0], "/dev/null");

my ($symrdf, $symsg, $symcli);

@EXPORT = qw(
);
$VERSION = '1.00';

require 5.000;

%default = (
        resname => '',
        name    => '',    #SRDF Storage Group Name
        sid   	=> '',
        rdfg   	=> '',
        home    => '',
        mode    => '',
        state   => '',
        role    => '',
        devs    => 0,
        invalid => 0,
        devfotime => 0,
        dcv_mode => '',
        onlinetimeout => 0,
        swaproles => 1,
        resource_type => 'SRDFSG',
        start_time => '',
        type => '-sg',
	consistency => 0,
        rdfgopt => '',
        is_vmware => 0,
        grp_type => 'sg',
        dev_state => '',
        srdf_state => '',
	symcli_ver => '0'                
 );


sub new {
    my($proto, $resname, $name, $symhome, $mode, $devfotime, $swaproles, $sid, $rdfg) = @_;
    my $class = ref($proto) || $proto;
    my $self = { %default };
    my (%arraydevs)  = ();
    my $func = "SRDF Storage Group::new";

    $self->{'resname'} = $resname;
    $self->{'name'} = $name;
    $self->{'sid'} = $sid;
    $self->{'rdfg'} = $rdfg;
    $self->{'home'} = $symhome;
    $self->{'mode'} = $mode;
    $self->{'swaproles'} = $swaproles;
    $self->{'start_time'} = $^T;
    $self->{'consistency'} = 0;

    if ($self->{'mode'} eq ''){
	    $self->{'mode'} = "MODE_NOT_SPEC";
    }

    my $osType = $^O;
    $self->{'home'} = $symhome;
    $symrdf = "$self->{'home'}/bin/symrdf";
    $symsg = "$self->{'home'}/bin/symsg";
    $symcli = "$self->{'home'}/bin/symcli";

    $self->{'devfotime'} = $devfotime;

    $self->{'rdfgopt'} = "";

    $self->{'symcli_ver'} = 0;	

    bless($self, $class);
    return $self;
}

sub DESTROY {}


sub get_count {
	my ($self) = @_;
	my $cur_time = time();
	my $func = "get_count";

# Get the resource-specific OnlineTimeout if set, else get the SRDF
# type specific value

	if ($self->{'onlinetimeout'} == 0) {
		my @onlinetimeout;
		if (run_cmd("$ag_i18n_inc::HARES -value $self->{'resname'} OnlineTimeout",
					\@onlinetimeout, 1, $func)) {
			run_cmd("$ag_i18n_inc::HATYPE -value $self->{'resource_type'} OnlineTimeout",
					\@onlinetimeout, $func);
		}
		$self->{'onlinetimeout'} = $onlinetimeout[0];
	}
	my $remaining_time = ($self->{'onlinetimeout'} - ($cur_time - $self->{'start_time'}));
	my $count = int($remaining_time / 5) - 1;
	VCSAG_LOGDBG_MSG(4, "$func: remaining_time: [$remaining_time]");
	VCSAG_LOGDBG_MSG(4, "$func: current time: [$cur_time]");
	VCSAG_LOGDBG_MSG(4, "$func: start_time: [$self->{'start_time'}]");
	VCSAG_LOGDBG_MSG(4, "$func: count value: [$count]");
	if ($count <= 0) {
		VCSAG_LOG_MSG("W", "not enough time to issue the symcli commands; administrative intervention or automatic restart required", 7);
		$self->_exit(0);
	}
	return $count;
}


sub failover_time {
	my ($self) = @_;
	my $func = "failover_time";
	my $cur_time = time();


	if ($self->{'onlinetimeout'} == 0) {
		my @onlinetimeout;
		if (run_cmd("$ag_i18n_inc::HARES -value $self->{'resname'} OnlineTimeout",
					\@onlinetimeout, 1, $func)) {
			run_cmd("$ag_i18n_inc::HATYPE -value $self->{'resource_type'} OnlineTimeout",
					\@onlinetimeout, $func);
		}
		$self->{'onlinetimeout'} = $onlinetimeout[0];
	}

	my $remaining_time = ($self->{'onlinetimeout'} - ($cur_time - $self->{'start_time'}));
	my $failover_time = $self->{'devs'} * $self->{'devfotime'};
	my $count = int(($remaining_time - ($failover_time + 10)) / 5);
	VCSAG_LOGDBG_MSG(4, "$func: remaining_time: [$remaining_time]");
	VCSAG_LOGDBG_MSG(4, "$func: failover_time: [$failover_time]");
	VCSAG_LOGDBG_MSG(4, "$func: current time: [$cur_time]");
	VCSAG_LOGDBG_MSG(4, "$func: start_time: [$self->{'start_time'}]");
	VCSAG_LOGDBG_MSG(4, "$func: count value: [$count]");
	return $count;
}

sub failover_required() {
	my ($self) = shift;
	my $splittakeover = shift;
	$self->get_srdf_state();

	if(($self->{'srdf_state'} eq "Invalid") || ($self->{'srdf_state'} eq "SuspendedWD")){
		VCSAG_LOGDBG_MSG(1, "Administration intervention required. state: $self->{'srdf_state'}");
		return $self->_exit(0);
	}

	if($self->{'srdf_state'} eq "SplitR2") {
		if ($splittakeover eq "0") {
			VCSAG_LOG_MSG("E", "devices in group $self->{'name'} are in Split state and SplitTakeover is set to 0; administrative intervention is required", 8, "$self->{'name'}");
			return $self->_exit(0);
		}
		VCSAG_LOG_MSG("N", "devices in group $self->{'name'} are in Split state and SplitTakeover is set to 1; no action is required", 9, "$self->{'name'}");
		return $self->online_resource(); 
	}

	if($self->{'srdf_state'} eq "RW") { 
		VCSAG_LOG_MSG("N", "devices in group $self->{'name'} are all read/write enabled; no action is required", 10, "$self->{'name'}");
		return $self->online_resource(); 
	}
	return ;
}

sub get_srdf_state {
	my ($self) = shift;
	my $func = "get_srdf_state";
	if (($self->{'state'} eq "Invalid") || ($self->{'state'} eq "")) {
		$self->{'srdf_state'} = "Invalid";
		VCSAG_LOG_MSG("E", "devices in group $self->{'name'} are in an invalid mix of writable or NR states; administrative intervention required", 11, "$self->{'name'}");
		return ; 
	}

	if (($self->{'state'} eq "RW") && ($self->{'role'} eq "r2")) {
		my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -split verify >$DEVNULL 2>&1", 0, 1, $func);
		if ($ret == 0) {
			$self->{'srdf_state'} = "SplitR2";
			return ; 
		}

		my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -partitioned verify >$DEVNULL 2>&1", 0, 1, $func);
		if ($ret == 0) {
			VCSAG_LOGDBG_MSG(1, "$func: Replication is in partition state ");
			$self->{'srdf_state'} = "ParitionR2RW";
			return ;
		}

	}
	if ($self->{'state'} eq "WD") {
		my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -suspended verify >$DEVNULL 2>&1", 0, 1, $func);
		if ($ret == 0) {
			$self->{'srdf_state'} = "SuspendedWD";
        VCSAG_LOG_MSG("E", "Devices in group $self->{'name'} are in suspended state. Administrative intervention required.", 12, "$self->{'name'}");

			return ;
		}
		$self->{'srdf_state'} = "Failover_req";
		return ;
	}
	$self->{'srdf_state'} = $self->{'state'};
	return ;
}

sub run_cmd {
	my ($cmd, $output_ref, $ignore_failure, $func) = @_;
	my $retval = 0;

	VCSAG_LOGDBG_MSG(1, "$func: Issuing command: [$cmd]");
	my $typ = ref($output_ref);
	if (!$typ) { # output is not a ref. So caller is ot interested in output.
		system($cmd);
		return $?;
	}

	my $cmd_txt = $cmd;
	if (1 == $ignore_failure) {
		$cmd_txt .= " 2>$DEVNULL";
	}
	if ($typ eq "ARRAY") {
		@{$output_ref} = `$cmd_txt`;
		chomp @{$output_ref};
	} else {
		$$output_ref = `$cmd_txt`;
		chomp ${$output_ref};
	}

	$retval = $?;
	if ($retval != 0) {
		unless (1 == $ignore_failure ) {
			$cmd_txt =~ s/\"//g;
			VCSAG_LOG_MSG("W", "Command: \'$cmd_txt\' failed !", 13, "$cmd_txt");
		}
	}
	return $retval;
}

sub takeover {
	my ($self, $autotakeover) = @_;
	my $count;
	my $failovercount;
	my $func = "takeover";

	if ($self->{'srdf_state'} eq "ParitionR2RW") {
		if($autotakeover != 1) {
			VCSAG_LOG_MSG("E", "devices in group $self->{'name'} are in Partitioned state but autotakeover is not enabled; administrative intervention is required", 50, "$self->{'name'}");
			$self->_exit(0);

		}
		VCSAG_LOG_MSG("N", "symrdf pair is in Paritioned. All devices in group $self->{'name'} are read/write enabled; autotakeover is enabled", 51, "$self->{'name'}");
		return $self->online_resource();
	}

	my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -partitioned verify > $DEVNULL 2>&1", 0, 1, $func);
	if($ret != 0 ) {
		VCSAG_LOGDBG_MSG(1, "$func: Replication is not in partition state");
		return ; #not in partition state return from here.
	}
	
	if($autotakeover != 1) {
		VCSAG_LOG_MSG("E", "devices in group $self->{'name'} are in Partitioned state but autotakeover is not enabled; administrative intervention is required", 44, "$self->{'name'}");
		$self->_exit(0);

	}

	VCSAG_LOG_MSG("N", "devices in group $self->{'name'} are in Partitioned state, and autotakeover is enabled; failover of devices will proceed", 45, "$self->{'name'}");

	$failovercount = $self->failover_time();
	if ($failovercount <= 0) {
		VCSAG_LOG_MSG("W", "not enough time to issue failover command; administrative intervention or automatic restart required", 46);
		$self->_exit(0);
	}

	my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -noprompt -c $failovercount -i 5 failover 2>&1", 0, 1, $func);
	if($ret) {
		VCSAG_LOG_MSG("E", "failover of Partitioned devices in group $self->{'name'} failed; administrative intervention is required", 47, "$self->{'name'}");
		$self->_exit(0);
	}

	$self->get_state();
	if ($self->{'state'} ne "RW") {
		VCSAG_LOG_MSG("E", "partition failover of group $self->{'name'} failed, devices are not read-write enabled", 48, "$self->{'name'}");
		$self->_exit(0);
	}

	VCSAG_LOG_MSG("N", "failover of group $self->{'name'} succeeded", 49, "$self->{'name'}");
	$self->online_resource(0);
}

sub failover {
	my ($self) = @_; 
	my $func = "failover";
	if($self->{'swaproles'}) {
		VCSAG_LOG_MSG("N", "swap role is enabled for group $self->{'name'}", 42, "$self->{'name'}");
		if($self->{'dcv_mode'} eq "Asynchronous") {
			$self->async_failover();
		}else {
			$self->failover_swap();
		}
	} else {
		VCSAG_LOG_MSG("N", "swap role is disabled for group $self->{'name'}", 43, "$self->{'name'}");
		$self->failover_no_swap();
	}
}

 
sub failover_swap {
	my ($self) = @_;
	my $count;
	my $halfCount;
	my $failovercount;

	my $ret = 0;
	my $func = "failover_swap";

   $count = $self->get_count();
	$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -c $count -i 5 -synchronized verify >$DEVNULL 2>&1", 0, 1, $func);

	if ($ret) {
		VCSAG_LOG_MSG("W", "all devices in group $self->{'name'} are not in synchronized or consistent state in preparation for failover; administrative intervention is required", 14, "$self->{'name'}");
		$self->_exit(0);
	}

    $failovercount = $self->failover_time();
    if ($failovercount <= 0) {
        VCSAG_LOG_MSG("W", "not enough time to issue failover command; administrative intervention or automatic restart required", 15);
        $self->_exit(0);
    }

#sync mode failover with role swap.
    VCSAG_LOG_MSG("N", "issuing failover/role swap of storage group $self->{'name'}", 15, "$self->{'name'}");
    my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -noprompt -i 5 -c $failovercount failover -establish 2>&1", $func);
    if($ret) {
	    VCSAG_LOG_MSG("E", "failover of device group $self->{'name'} failed; error code is $ret", 16, "$self->{'name'}", "$ret");
	    $self->_exit(0);
    }
    sleep(3);

    $self->get_state();

    if ($self->{'state'} ne "RW") {
	    VCSAG_LOG_MSG("E", "failover of group $self->{'name'} failed, devices are not read-write enabled", 17, "$self->{'name'}");
	    $self->_exit(0);
    }

    VCSAG_LOG_MSG("N", "failover of devices in group $self->{'name'} succeeded", 18, "$self->{'name'}");
    $self->online_resource(0);

}

sub failover_no_swap {
	my($self) = shift;
	my $func = "failover_no_swap";
	my $failovercount = 0;


	$failovercount = $self->failover_time();
	if ($failovercount <= 0) {
		VCSAG_LOG_MSG("W", "not enough time to issue failover command; administrative intervention or automatic restart required", 38);
		$self->_exit(0);
	}

	VCSAG_LOG_MSG("N", "issuing failover of group $self->{'name'}", 37, "$self->{'name'}");

	my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -noprompt -i 5 -c $failovercount failover 2>&1",0, 1, $func);
	if($ret) {
		VCSAG_LOG_MSG("E", "failover of group $self->{'name'} failed; error code is $ret", 39, "$self->{'name'}", "$ret");
		$self->_exit(0);
	}

	$self->get_state();
	if ($self->{'state'} ne "RW") {
		VCSAG_LOG_MSG("E", "failover of group $self->{'name'} failed, devices are not read-write enabled", 40, "$self->{'name'}");
		$self->_exit(0);
	}

	VCSAG_LOG_MSG("N", "failover of group $self->{'name'} succeeded", 41, "$self->{'name'}");
	$self->online_resource(0);
}

sub unlock_res {
	my($self) = shift;

	my ($grp) = split('\n', `$ag_i18n_inc::HARES -value $self->{'resname'} Group`);
	my ($para) = split('\n', `$ag_i18n_inc::HAGRP -value $grp Parallel`);
	return 0 if ($para != 1);

	`$ag_i18n_inc::HARES -unlock $self->{'resname'} > /dev/null 2>&1`;
}

sub _exit {
	my ($self, $stat) = @_;
	$self->unlock_res();
	exit($stat);
}

sub get_state {
	my $self = shift;
	my $ret;
	my $indexRDFG = 0;
	my $indexLSymmID = 0;
	my $indexRSymmID = 0;
	my $key;
	my $devidx = 0;
	my $output;
	my $func = "get_state";
	my $valid_mode = 0;
	my $cmd = "$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} query";
	run_cmd($cmd, \$output, 1, $func);
	my @out;
	my $modepat= qr/[SAD][X\.]/;

	if($self->{'symcli_ver'}<9.0){
		$modepat= qr/[SAC]\./;
	}

	if(!$output) {
		$ret = "$?";
		my $cmd_txt=$cmd;
		$cmd_txt =~ s/\"//g;
		VCSAG_LOG_MSG("E", "Command $cmd_txt failed with error code:$ret\n", 18, "$cmd_txt", "$ret");
		$self->_exit(0);
	}

	my @query = split('\n', $output);
	$self->{'state'} = "";
	foreach my $q (@query) {
		if ($q =~ /Group Number/) {  
			next;
		}
		if ($q =~ /^Remote Symmetrix ID/) {
			next;
		}

		if ($q =~ /Source/) {
			my @roleinfo = split('\s+', $q);
			if ($roleinfo[1] eq "Source") {
				VCSAG_LOGDBG_MSG(3, "Storage group $self->{'name'} has R1 role ");
				$self->{'role'} = "r1";
			} else {
				VCSAG_LOGDBG_MSG(3, "Storage group $self->{'name'} has R2 role ");
				$self->{'role'} = "r2";
			}
			next;
		}


		if(!($q =~ /$modepat/)) {
			next;
		}

		my @stateinfo = split('\s+', $q);

		if ($stateinfo[10] =~ /S/) {
			$valid_mode = 1;
			$self->{'dcv_mode'} = "Synchronous";
			VCSAG_LOGDBG_MSG(4, "MACE Sync mode of operation");
		} elsif ($stateinfo[10] =~ /A/) {
			$valid_mode = 1;
			$self->{'dcv_mode'} = "Asynchronous";
			VCSAG_LOGDBG_MSG(4, "MACE Async mode of operation");
		}


		if ($stateinfo[2] eq "RW") {
			if ($self->{'state'} eq "WD") {
				$self->{'state'} = "Invalid";
				VCSAG_LOGDBG_MSG(1, "Invalid state set");
				return;
			} else {
				$self->{'state'} = "RW";
			}
		}
		if ($stateinfo[2] eq "WD") {
			if ($self->{'state'} eq "RW") {
				$self->{'state'} = "Invalid";
				VCSAG_LOGDBG_MSG(1, "Invalid state set.");
				return;
			} else {
				$self->{'state'} = "WD";
			}
		} 	
	}

	if ($valid_mode != 1) {
		VCSAG_LOGDBG_MSG(1, "Only Synchronous and Asynchronous mode is supported.");
		$self->_exit(0);
	}
}

sub vxvm_scan {
	my ($self) = @_;
	my $func = "vxvm_scan";
	if ($OS eq "Windows NT") {
		return;
	}
#Required logic for selective scan disk
	run_cmd("$VXDISK scandisks", 0, 1, $func);
}

sub wait_for_update {
	my $self = shift;
	my $count = shift;
	my $func = "wait_for_update";
	my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -updateinprog verify >$DEVNULL 2>&1", 0, 1, $func);
	$ret = $ret / 256;
	if ($ret != 36) {
		VCSAG_LOG_MSG("N", "failback of group $self->{'name'} will wait until all updates in progress complete or entry point times out", 75, "$self->{'name'}");
		my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -c $count -i 5 -updated verify >$DEVNULL 2>&1", 0, 1, $func);
		if ($ret) {
			return 1;
		}
	}

	return 0;
}

sub wait_for_sync {
	my ($self) = @_;
	my $count = $self->get_count();
	my $halfCount;
	my $ret;
	my $func = "wait_for_sync";
#      1) check for async configuration.

	if ($self->{'role'} eq "r1") {
		VCSAG_LOGDBG_MSG(1, "Group or Role is R1 type.");
		$ret = $self->wait_for_update($count);
		return $ret;
	}	

	$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -syncinprog verify >$DEVNULL 2>&1", 0, 1, $func);
	$ret = $ret / 256;
	if ($ret != 28) {
		VCSAG_LOG_MSG("N", "failover of group $self->{'name'} will wait until all synchronization in progress complete or entry point times out", 19, "$self->{'name'}");

		if ($self->{'dcv_mode'} eq "Synchronous") {
			VCSAG_LOG_MSG("N", "failover of group $self->{'name'} will wait until all synchronization in progress complete or entry point times out", 19, "$self->{'name'}");
			$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -c $count -i 5 -synchronized verify >$DEVNULL 2>&1", 0, 1, $func);
		}else{
			$halfCount = int($count / 2);
			$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'}  -c $halfCount -i 5 -consistent verify >$DEVNULL 2>&1", 0, 1, $func);
			if ($ret) {
				$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -c $halfCount -i 5 -synchronized verify >$DEVNULL 2>&1", 0, 1, $func);
			}
		}

		if ($ret) {
			VCSAG_LOGDBG_MSG(1, "synchronize verify command return with [$ret]");
			return 1;
		}
	}
	return 0;
}


sub link_monitor_r1 {
	my $self = shift;
	my $linkmonitor = shift;
	my $ret;
	my $func = "link_monitor_r1";
	if($self->{'state'} ne "RW") {
		VCSAG_LOGDBG_MSG(1, "Devices are not in RW state.");
		return ;
	}

	if($self->{'role'} ne "r1") {
		VCSAG_LOGDBG_MSG(1, "Group or Role is not R1 type.");
		return ;
	}

	if(!$linkmonitor) {
		VCSAG_LOG_MSG("N", "devices in group $self->{'name'} are all read/write enabled; LinkMonitor is set to [$linkmonitor]. no action is required. ", 20, "$self->{'name'}", "$linkmonitor");

		return $self->online_resource();
	}

	my $count = $self->get_count();

	if($self->{'dcv_mode'} eq "Synchronous") {
		$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -c $count -i 5 -synchronized verify >$DEVNULL 2>&1", 0, 1, $func);

	}elsif($self->{'dcv_mode'} eq "Asynchronous"){
		my $halfCount = int($count / 2);
		$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -c $halfCount -i 5 -consistent verify >$DEVNULL 2>&1", 0, 1, $func);
		if($ret){
			$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -c $halfCount -i 5 -synchronized verify >$DEVNULL 2>&1", 0, 1, $func);
		}
	}

	if ($ret) {
		VCSAG_LOG_MSG("E", "Replication link is neither in synchronized nor in consistent state. LinkMonitor is set to [$linkmonitor]. Administrative intervention is required.", 21, "$linkmonitor");
		return $self->_exit(0);
	}

	VCSAG_LOG_MSG("N", "devices in group $self->{'name'} are all read/write enabled; no action is required", 22, "$self->{'name'}");
	return $self->online_resource();
}

sub online_resource {
	my $self = shift;
	$self->vxvm_scan();
	VCSAG_LOGDBG_MSG(1, "creating lock file ");
	$self->create_lockfile();
	$self->cvm_import();
#CVM import deport action entry point.
	$self->_exit(0);
}

sub failback {
	my ($self) = @_;
	my $count;
	my $halfCount;
	my $failbackcount;
	my $func = "failback";
#
#Get the resource-specific OnlineTimeout if set, else get the SRDF
#type specific value
#                        
	if ($self->{'onlinetimeout'} == 0) {
		my @onlinetimeout;
		if (run_cmd("$ag_i18n_inc::HARES -value $self->{'resname'} OnlineTimeout",
					\@onlinetimeout, 1, $func)) {
			run_cmd("$ag_i18n_inc::HATYPE -value $self->{'resource_type'} OnlineTimeout",
					\@onlinetimeout, $func);
		}
		$self->{'onlinetimeout'} = $onlinetimeout[0];
	}

	$count = $self->get_count();
	$halfCount = int($count / 2);
	my $ret;
	my $temp_counter = 1;               # imp to exit on temp_counter = half counter

		if($halfCount == 0) {
#Try to execute failedover and updated verification atleast once
			$halfCount = 1;
		}

	while ($temp_counter <= $halfCount) {
		VCSAG_LOGDBG_MSG(3, "Counter no: $temp_counter for failedover and updated verification");
		my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -failedover verify > $DEVNULL 2>&1", 0, 1, $func);
		if ($ret) {
			$count = $self->get_count();
			$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -updated verify > $DEVNULL 2>&1", 0, 1, $func);
			if ($ret) {
				if($temp_counter == $halfCount) {
					VCSAG_LOG_MSG("E", "all devices in group $self->{'name'} are not in Failed Over or Updated state in preparation for failback; administrative intervention is required", 23, "$self->{'name'}");
					$self->_exit(0);
				}
				sleep (5);
				$temp_counter = $temp_counter + 1;
				next;
			}
		}
		last;
	}

	$failbackcount = $self->failover_time();
	if ($failbackcount <= 0) {
		VCSAG_LOG_MSG("W", "not enough time to issue failback command; administrative intervention or automatic restart required", 24);
		$self->_exit(0);
	}

	VCSAG_LOG_MSG("N", "issuing failback of group $self->{'name'}", 24, "$self->{'name'}");
	$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -noprompt -i 5 -c $failbackcount failback", 0, 1, $func);

	if ($ret) {
		$ret = $ret / 256;
		VCSAG_LOG_MSG("E", "failback of group $self->{'name'} failed; error code is $ret", 25, "$self->{'name'}", "$ret");
		$self->_exit(0);
	}

	$self->get_state();
	if ($self->{'state'} eq "RW") {
		VCSAG_LOG_MSG("N", "failback of devices in group $self->{'name'} succeeded", 26, "$self->{'name'}");
		$self->online_resource();	
	} else {
		VCSAG_LOG_MSG("E", "failback of group $self->{'name'} failed", 27, "$self->{'name'}");
		$self->_exit(0);
	}
}

sub get_lockfilename {
    my ($self) = shift;
    my $typ;
    my $lock_file;
    my $lock_dir = "";

    if (!defined($self->{'resource_type'}) ||
            (length($self->{'resource_type'}) == 0)) {

        $typ = `$ag_i18n_inc::HARES -value $self->{'resname'} Type`;
        chomp $typ;
    } else {
        $typ = $self->{'resource_type'};
    }

    if ($^O =~ /MSWin32/i) {
        $lock_dir = $ag_i18n_inc::VCSLOG . "\\lock\\" . $typ . "\\";
    } else {
        $lock_dir = $ag_i18n_inc::VCSLOG . "/lock/" . $typ . "/";
    }

    $lock_file = $lock_dir . ".VCS_" . $typ . "_" . $self->{'resname'} .
        "_Online_Lockfile.DO-NOT-REMOVE";

    return $lock_file;
}

sub create_lockfile {
    my ($self) = shift;
    my ($lock_file) = $self->get_lockfilename();

    my (undef, $lock_dir, undef) = fileparse($lock_file);

    if (! -d $lock_dir) {
        mkdir  $lock_dir or
            VCSAG_LOG_MSG("E", "Could not create lock directory $lock_dir",
                    28, "$lock_dir");
    }

    open(LFH, ">$lock_file") or
        VCSAG_LOG_MSG("E", "Could not create file: $lock_file", 29,
                "$lock_file");

    close(LFH);
}


sub remove_lockfile {
    my($self) = shift;
    my ($lock_file) = $self->get_lockfilename();

    if (-f "$lock_file") {
        unlink("$lock_file") or
            VCSAG_LOG_MSG("E", "Could not delete file: $lock_file", 30,
                    "$lock_file");
    }
}


sub exists_lockfile {
    my ($self) = shift;
    my $retval = 0;
    my ($lock_file) = $self->get_lockfilename();

    if (-f "$lock_file") {
        $retval = 1;
    }
    return $retval;
}


sub getAdvOpts {
    shift;
    my @args = @_;
    my ( $ret, $numAdOpts, $advOptsIndex, $loop, %advOpts );

    ( $ret, $numAdOpts, $advOptsIndex ) = VCSAG_GET_ATTR_VALUE ( 'AdvancedOpts', @args );

    for ( $loop = 1; $loop <= $numAdOpts; $loop++ ) {
        my ($key, $value);
        ($ret, $key) = VCSAG_GET_ATTR_VALUE ('AdvancedOpts', $advOptsIndex, $loop, @ARGV);
        $loop++;
        ($ret, $value) = VCSAG_GET_ATTR_VALUE ('AdvancedOpts', $advOptsIndex, $loop, @ARGV);
        $advOpts {$key} = $value;
    }
    return %advOpts;
}

sub async_failover {
	my($self) = shift;
	my $func = "async_failover";
	my $failovercount = 0;

	my $count = $self->get_count();

	my $halfCount = int($count / 2);
	my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -c $halfCount -i 5 -consistent verify > $DEVNULL 2>&1", 0, 1, $func);       

	if ($ret) {
		$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -c $halfCount -i 5 -synchronized verify >$DEVNULL 2>&1", 0, 1, $func);  
	}

	if ($ret) {
		VCSAG_LOG_MSG("W", "all devices in group $self->{'name'} are not in synchronized or consistent state in preparation for failover; administrative intervention is required", 50, "$self->{'name'}");
		$self->_exit(0);
	}

	my $failovercount = $self->failover_time();
	if ($failovercount <= 0) {
		VCSAG_LOG_MSG("W", "not enough time to issue failover command; administrative intervention or automatic restart required", 51);
		$self->_exit(0);
	}

	VCSAG_LOG_MSG("N", "issuing -force failover of storage group $self->{'name'}", 52, "$self->{'name'}");

	my $ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -noprompt -force -i 5 -c $failovercount failover 2>&1", $func);
	if($ret) {
		VCSAG_LOG_MSG("E", "failover of storage group $self->{'name'} failed; error code is $ret", 53, "$self->{'name'}", "$ret");
		$self->_exit(0);
	}

	$count = $self->get_count();
	$self->get_consistency();
	if ($self->{'consistency'}) {
		VCSAG_LOG_MSG("N", "disabling consistency for storage group $self->{'name'}", 54, "$self->{'name'}");
		$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -i 5 -c $count -noprompt disable >$DEVNULL", 0, 1, $func);
		if ($ret) {
			VCSAG_LOG_MSG("E", "Disabling the consistency for the device group $self->{'name'} failed; error code is $ret", 55, "$self->{'name'}", "$ret");
			$self->_exit(0);
		}
	}
	sleep(3);
	$count = $self->get_count();
	VCSAG_LOG_MSG("N", "issuing role swap of storage group $self->{'name'}", 56, "$self->{'name'}");
	$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -noprompt -i 5 -c $count swap >$DEVNULL", $func);
	if($ret) {
		VCSAG_LOG_MSG("E", "Role-swap of the device group $self->{'name'} failed; error code is $ret", 57, "$self->{'name'}", "$ret");
		$self->_exit(0);
	}

	$count = $self->get_count();
	VCSAG_LOG_MSG("N", "issuing establish/resume of storage group $self->{'name'}", 58, "$self->{'name'}");
	$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -noprompt -i 5 -c $count establish", $func);
	if($ret) {
		VCSAG_LOG_MSG("E", "Establish=link for the storage group $self->{'name'} failed; error code is $ret", 59, "$self->{'name'}", "$ret");
		$self->_exit(0);
	}

	$count = $self->get_count();
	if ($self->{'consistency'}) {
		VCSAG_LOG_MSG("N", "re-enable consistency for storage group $self->{'name'}", 60, "$self->{'name'}");
		$ret = run_cmd("$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} -noprompt -i 5 -c $count enable >$DEVNULL", 0, 1, $func);
		if ($ret) {
			VCSAG_LOG_MSG("N", "re-enabling consistency on storage group $self->{'name'} failed. Administrative intervention required.", 61, "$self->{'name'}");
			$self->_exit(0);
		}
	}

	sleep(3);
	$self->get_state();

	if ($self->{'state'} ne "RW") {
		VCSAG_LOG_MSG("E", "failover of group $self->{'name'} failed, devices are not read-write enabled", 62, "$self->{'name'}");
		$self->_exit(0);
	}

	VCSAG_LOG_MSG("N", "failover of devices in group $self->{'name'} succeeded", 63, "$self->{'name'}");
	$self->online_resource(0);

}
                                                                                
sub get_consistency {
	my $self = shift;
	my $ret;
	my $output;
	my $func = "get_consistency";
	my $cmd = "$symrdf $self->{'type'} $self->{'name'} -sid $self->{'sid'} -rdfg $self->{'rdfg'} $self->{'rdfgopt'} query -rdfa";
	run_cmd($cmd, \$output, 1, $func);
	my @out;
	my $modepat= qr/[SAD][X\.]/;

        if($self->{'symcli_ver'}<9.0){
                $modepat= qr/[SAC]\./;
        }

	if(!$output) { 
		$ret = "$?";
		my $cmd_txt=$cmd;
		$cmd_txt =~ s/\"//g;
		VCSAG_LOG_MSG("E", "Command $cmd_txt failed with error code:$ret\n", 18, "$cmd_txt", "$ret");
		return;
	}

	my @query = split('\n', $output);
	$self->{'state'} = "";
	foreach my $q (@query) {
	#TODO: Only look for MACE flags value, optimize the logic in later phase.
		if ($q =~ /Group Number/) { 
			next;
		}
		if ($q =~ /^Remote Symmetrix ID/) {
			next;
		}

		if ($q =~ /Source/) {
			next;
		}

		if(!($q =~ /$modepat/)) {
			next;
		}

		my @stateinfo = split('\s+', $q);

		VCSAG_LOGDBG_MSG(1, "Checking for consistency state");
		my @maceflags = split(//, $stateinfo[10]);
		if($self->{'symcli_ver'}<9.0) {
			if ($maceflags[2] =~ /X/) {
				VCSAG_LOGDBG_MSG(1, "Consistency is Enabled");
				$self->{'consistency'} = 1;
			} 
			elsif ($maceflags[2] =~ /./) {
				VCSAG_LOGDBG_MSG(1, "Consistency is Disabled");
				$self->{'consistency'} = 0;
			}
		}
		else {
			if ($maceflags[1] =~ /X/) {
				VCSAG_LOGDBG_MSG(1, "Consistency is Enabled");
				$self->{'consistency'} = 1;
			} 
			elsif ($maceflags[1] =~ /./) {
				VCSAG_LOGDBG_MSG(1, "Consistency is Disabled");
				$self->{'consistency'} = 0;
			}
		}	
	}	
}

sub extract_CLI_version{
	my ($self) = shift;
	my @tmparr = ();
	my $func = "extract_CLI_version";

	run_cmd("$symcli", \@tmparr, 1, $func);
	my $version = "";
	my $pattern= qr/(\d.\d.\d)/;
	foreach(@tmparr) {
		my $out1 = "";
		$out1 = $_;
		if(($out1 =~ /Symmetrix Command/) && ($out1 =~ /$pattern/)) {
			$self->{'symcli_ver'}=$1;
			last;
		}	
	} 
}



